/* ------------------------------------------------------------------------- */
/*
 *  adjust.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Thu 26 Apr 2007 19:55:04 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_ADJUST_H
#define CLX_ADJUST_H

#include <string>

namespace clx {
	/* --------------------------------------------------------------------- */
	//  ljust
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& ljust(std::basic_string<CharT, Traits>& s,
		unsigned int n, CharT c = ' ') {
		if (s.length() < n) s.append(n - s.length(), c);
		return s;
	}
	
	/* --------------------------------------------------------------------- */
	//  ljust_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> ljust_copy(const std::basic_string<CharT, Traits>& s,
		unsigned int n, CharT c = ' ') {
		std::basic_string<CharT, Traits> tmp(s);
		return ljust(tmp, n, c);
	}
	
	/* --------------------------------------------------------------------- */
	//  rjust
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& rjust(std::basic_string<CharT, Traits>& s,
		unsigned int n, CharT c = ' ') {
		if (s.length() < n) s.insert(s.begin(), n - s.length(), c);
		return s;
	}
	
	/* --------------------------------------------------------------------- */
	//  rjust_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> rjust_copy(const std::basic_string<CharT, Traits>& s,
		unsigned int n, CharT c = ' ') {
		std::basic_string<CharT, Traits> tmp(s);
		return rjust(tmp, n, c);
	}
	
	/* --------------------------------------------------------------------- */
	//  center
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& center(std::basic_string<CharT, Traits>& s,
		unsigned int n, CharT c = ' ') {
		if (s.length() < n) {
			unsigned int quo = (n - s.length()) / 2;
			unsigned int odd = (n - s.length()) % 2;
			ljust(s, quo + s.length(), c);
			if (odd != 0) quo++;
			rjust(s, quo + s.length(), c);
		}
		return s;
	}
	
	/* --------------------------------------------------------------------- */
	//  center_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> center_copy(const std::basic_string<CharT, Traits>& s,
		unsigned int n, CharT c = ' ') {
		std::basic_string<CharT, Traits> tmp(s);
		return center(tmp, n, c);
	}
}

#endif // CLX_ADJUST_H
