/* ------------------------------------------------------------------------- */
/*
 *  predicate.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Mon 23 Apr 2007 18:34:15 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_PREDICATE_H
#define CLX_PREDICATE_H

#include <locale>
#include <set>

namespace clx {
	/* --------------------------------------------------------------------- */
	//  classified_functor
	/* --------------------------------------------------------------------- */
	class classified_functor {
	public:
		explicit classified_functor(std::ctype_base::mask m, const std::locale& loc = std::locale()) :
			mask_(m), loc_(loc) {}
		
		template <class CharT>
		bool operator()(CharT c) const {
			return std::use_facet<std::ctype<CharT> >(loc_).is(mask_, c);
		}
		
	private:
		const std::ctype_base::mask mask_;
		const std::locale loc_;
	};
	
	/* --------------------------------------------------------------------- */
	//  charset_functor
	/* --------------------------------------------------------------------- */
	template <class CharT>
	class charset_functor {
	public:
		typedef CharT char_type;
		typedef unsigned int size_type;
		
		explicit charset_functor(const char_type* s) : charset_() {
			while (s != NULL && *s != '\0') charset_.insert(*s++);
		}
		
		bool operator()(char_type c) const {
			return charset_.find(c) != charset_.end();
		}
		
	private:
		std::set<char_type> charset_;
	};
	
	/* --------------------------------------------------------------------- */
	//  is_any_of
	/* --------------------------------------------------------------------- */
	template <class CharT>
	charset_functor<CharT> is_any_of(const CharT* s) {
		return charset_functor<CharT>(s);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_space
	/* --------------------------------------------------------------------- */
	classified_functor is_space(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::space, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_alnum
	/* --------------------------------------------------------------------- */
	classified_functor is_alnum(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::alnum, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_alpha
	/* --------------------------------------------------------------------- */
	classified_functor is_alpha(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::alpha, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_cntrl
	/* --------------------------------------------------------------------- */
	classified_functor is_cntrl(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::cntrl, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_digit
	/* --------------------------------------------------------------------- */
	classified_functor is_digit(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::digit, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_graph
	/* --------------------------------------------------------------------- */
	classified_functor is_graph(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::graph, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_lower
	/* --------------------------------------------------------------------- */
	classified_functor is_lower(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::lower, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_upper
	/* --------------------------------------------------------------------- */
	classified_functor is_upper(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::upper, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_print
	/* --------------------------------------------------------------------- */
	classified_functor is_print(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::print, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_punct
	/* --------------------------------------------------------------------- */
	classified_functor is_punct(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::punct, loc);
	}
	
	/* --------------------------------------------------------------------- */
	//  is_xdigit
	/* --------------------------------------------------------------------- */
	classified_functor is_xdigit(const std::locale& loc = std::locale()) {
		return classified_functor(std::ctype_base::xdigit, loc);
	}
}

#endif // CLX_PREDICATE_H
