/* ------------------------------------------------------------------------- */
/*
 *  remove.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Thu 26 Apr 2007 17:55:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_REMOVE_H
#define CLX_REMOVE_H

#include <string>
#include <algorithm>
#include <functional>
#include "predicate.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  remove_if
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits, class PredicateFunc>
	std::basic_string<CharT, Traits>& remove_if(
		std::basic_string<CharT, Traits>& s, PredicateFunc f) {
		typename std::basic_string<CharT, Traits>::iterator last;
		last = std::remove_if(s.begin(), s.end(), f);
		if (last != s.end()) s.erase(last, s.end());
		return s;
	}
	
	/* --------------------------------------------------------------------- */
	//  remove_copy_if
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits, class PredicateFunc>
	std::basic_string<CharT, Traits> remove_copy_if(
		const std::basic_string<CharT, Traits>& s, PredicateFunc f) {
		std::basic_string<CharT, Traits> tmp(s);
		return remove_if(tmp, f);
	}
	
	/* --------------------------------------------------------------------- */
	//  remove
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& remove(
		std::basic_string<CharT, Traits>& s, CharT c) {
		return remove_if(s, std::bind2nd(std::equal_to<CharT>(), c));
	}
	
	/* --------------------------------------------------------------------- */
	//  remove_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> remove_copy(
		const std::basic_string<CharT, Traits>& s, CharT c) {
		std::basic_string<CharT, Traits> tmp(s);
		return remove(tmp, c);
	}
	
	/* --------------------------------------------------------------------- */
	//  unique
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& unique(std::basic_string<CharT, Traits>& s) {
		typename std::basic_string<CharT, Traits>::iterator last;
		last = std::unique(s.begin(), s.end());
		if (last != s.end()) s.erase(last, s.end());
		return s;
	}
	
	/* --------------------------------------------------------------------- */
	//  unique_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> unique_copy(const std::basic_string<CharT, Traits>& s) {
		std::basic_string<CharT, Traits> tmp(s);
		return unique(tmp);
	}
	
	/* --------------------------------------------------------------------- */
	//  squeeze_if
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits, class PredicateFunc>
	std::basic_string<CharT, Traits>& squeeze_if(
		std::basic_string<CharT, Traits>& s, PredicateFunc f) {
		bool x = false;
		typename std::basic_string<CharT, Traits>::iterator pos = s.begin();
		while (pos != s.end()) {
			if (f(*pos)) {
				if (x) pos = s.erase(pos);
				else {
					x = true;
					pos++;
				}
			}
			else {
				x = false;
				pos++;
			}
		}
		
		return s;
	}
	
	/* --------------------------------------------------------------------- */
	//  squeeze_copy_if
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits, class PredicateFunc>
	std::basic_string<CharT, Traits> squeeze_copy_if(
		const std::basic_string<CharT, Traits>& s, PredicateFunc f) {
		std::basic_string<CharT, Traits> tmp(s);
		return squeeze_if(tmp, f);
	}
	
	/* --------------------------------------------------------------------- */
	//  squeeze
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& squeeze(
		std::basic_string<CharT, Traits>& s, CharT c) {
		return squeeze_if(s, std::bind2nd(std::equal_to<CharT>(), c));
	}
	
	/* --------------------------------------------------------------------- */
	//  squeeze_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> squeeze_copy(
		const std::basic_string<CharT, Traits> s, CharT c) {
		std::basic_string<CharT, Traits> tmp(s);
		return squeeze(tmp, c);
	}
}
#endif // CLX_REMOVE_H
