/* ------------------------------------------------------------------------- */
/*
 *  replace.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Thu 26 Apr 2007 19:25:08 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_REPLACE_H
#define CLX_REPLACE_H

#include <string>

namespace clx {
	/* --------------------------------------------------------------------- */
	//  replace
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& replace(std::basic_string<CharT, Traits>& s,
		const std::basic_string<CharT, Traits>& sch,
		const std::basic_string<CharT, Traits>& rep,
		unsigned int nth = 1) {
		unsigned int i = 0;
		typename std::basic_string<CharT, Traits>::size_type pos = 0;
		while ((pos = s.find(sch, pos)) != std::basic_string<CharT, Traits>::npos) {
			if (++i == nth) {
				s.replace(pos, sch.length(), rep);
				break;
			}
			pos += sch.length();
		}
		return s;
	}
	
	/* --------------------------------------------------------------------- */
	//  replace
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& replace(std::basic_string<CharT, Traits>& s,
		const CharT* sch, const CharT* rep, unsigned int nth = 1) {
		std::basic_string<CharT, Traits> sch_tmp(sch);
		std::basic_string<CharT, Traits> rep_tmp(rep);
		return replace(s, sch_tmp, rep_tmp, nth);
	}
	
	/* --------------------------------------------------------------------- */
	//  replace_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> replace_copy(
		const std::basic_string<CharT, Traits>& s,
		const std::basic_string<CharT, Traits>& sch,
		const std::basic_string<CharT, Traits>& rep,
		unsigned int nth = 1) {
		std::basic_string<CharT, Traits> tmp(s);
		return replace(tmp, sch, rep, nth);
	}
	
	/* --------------------------------------------------------------------- */
	//  replace_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> replace_copy(
		const std::basic_string<CharT, Traits>& s,
		const CharT* sch, const CharT* rep, unsigned int nth = 1) {
		std::basic_string<CharT, Traits> tmp(s);
		return replace(tmp, sch, rep, nth);
	}
	
	/* --------------------------------------------------------------------- */
	//  replace_all
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& replace_all(std::basic_string<CharT, Traits>& s,
		const std::basic_string<CharT, Traits>& sch,
		const std::basic_string<CharT, Traits>& rep) {
		typename std::basic_string<CharT, Traits>::size_type pos = 0;
		while ((pos = s.find(sch, pos)) != std::basic_string<CharT, Traits>::npos) {
			s.replace(pos, sch.length(), rep);
			pos += rep.length();
		}
		return s;
	}
	
	/* --------------------------------------------------------------------- */
	//  replace_all
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits>& replace_all(std::basic_string<CharT, Traits>& s,
		const CharT* sch, const CharT* rep) {
		std::basic_string<CharT, Traits> sch_tmp(sch);
		std::basic_string<CharT, Traits> rep_tmp(rep);
		return replace_all(s, sch_tmp, rep_tmp);
	}
	
	/* --------------------------------------------------------------------- */
	//  replace_all_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> replace_all_copy(
		const std::basic_string<CharT, Traits>& s,
		const std::basic_string<CharT, Traits>& sch,
		const std::basic_string<CharT, Traits>& rep) {
		std::basic_string<CharT, Traits> tmp(s);
		return replace_all(tmp, sch, rep);
	}
	
	/* --------------------------------------------------------------------- */
	//  replace_all_copy
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits>
	std::basic_string<CharT, Traits> replace_all_copy(
		const std::basic_string<CharT, Traits>& s,
		const CharT* sch, const CharT* rep) {
		std::basic_string<CharT, Traits> tmp(s);
		return replace_all(tmp, sch, rep);
	}
}

#endif // CLX_REPLACE_H
