/* ------------------------------------------------------------------------- */
/*
 *  split.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Thu 26 Apr 2007 19:12:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_SPLIT_H
#define CLX_SPLIT_H

#include <iterator>
#include "predicate.h"
#include "lexical_cast.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  split_if
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits, class Container, class PredicateFunc>
	Container& split_if(const std::basic_string<CharT, Traits>& s, Container& result,
		PredicateFunc f, bool emtok = false) {
		typedef typename Container::value_type value_type;
		typedef typename std::basic_string<CharT, Traits>::const_iterator const_iterator;
		
		std::basic_string<CharT, Traits> tmp;
		std::insert_iterator<Container> it(result, result.begin());
		for (const_iterator pos = s.begin(); pos != s.end(); pos++) {
			if (!f(*pos)) tmp += *pos;
			else if (emtok || !tmp.empty()) {
				*it++ = lexical_cast<value_type>(tmp);
				tmp.erase();
			}
		}
		if (!tmp.empty()) *it++ = lexical_cast<value_type>(tmp);
		
		return result;
	}
	
	/* --------------------------------------------------------------------- */
	//  split
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits, class Container>
	Container& split(const std::basic_string<CharT, Traits>& s, Container& result,
		bool emtok = false, const std::locale& loc = std::locale()) {
		return split_if(result, s, is_space(loc), emtok);
	}
	
	/* --------------------------------------------------------------------- */
	//  join
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits, class Container>
	std::basic_string<CharT, Traits>& join(const Container& v,
		std::basic_string<CharT, Traits>& result, const std::basic_string<CharT, Traits>& delim) {
		typename Container::const_iterator pos = v.begin();
		if (pos != v.end()) result += *pos++;
		while (pos != v.end()) {
			result += delim;
			result += *pos++;
		}
		return result;
	}
	
	/* --------------------------------------------------------------------- */
	//  join
	/* --------------------------------------------------------------------- */
	template <class CharT, class Traits, class Container>
	std::basic_string<CharT, Traits>& join(const Container& v,
		std::basic_string<CharT, Traits>& result, const CharT* delim) {
		std::basic_string<CharT, Traits> tmp(delim);
		return join(v, result, tmp);
	}
}

#endif // CLX_SPLIT_H
