/* ------------------------------------------------------------------------- */
/*
 *  table.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Thu 30 Aug 2007 11:32:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TABLE_H
#define CLX_TABLE_H

#include <string>
#include <vector>
#include <istream>
#include <fstream>
#include <stdexcept>
#include "tokenizer.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  table
	/* --------------------------------------------------------------------- */
	template <
		class TokenizerFunc,
		class Type = std::string,
		template <class> class Container = std::vector
	>
	class table : public Container<Container<Type> > {
	public:
		typedef Type value_type;
		typedef TokenizerFunc functor;
		typedef Container<Container<Type> > container;
		typedef basic_tokenizer<TokenizerFunc, Type> parser;
		typedef typename parser::char_type char_type;
		typedef typename parser::string_type string_type;
		typedef std::basic_istream<char_type> istream;
		
		// constructor and destructor
		explicit table(const functor& f = functor()) :
			container(), parser_(f) { this->reset(); }
		
		explicit table(istream& sin, const functor& f = functor()) :
			container(), parser_(f) {
			this->reset();
			this->read(sin);
		}
		
		explicit table(const char_type* path, const functor& f = functor()) :
			container(), parser_(f) {
			this->reset();
			this->read(path);
		}
		
		explicit table(const string_type& path, const functor& f = functor()) :
			container(), parser_(f) {
			this->reset();
			this->read(path);
		}
		
		virtual ~table() {}
		
		// operator
		virtual void reset() {
			this->clear();
			parser_.reset();
		}
		
		virtual bool readline(istream& sin) {
			string_type tmp;
			if (!std::getline(sin, tmp)) return false;
			this->insert(this->end(), parser_(tmp));
			return true;
		}
		
		table& read(istream& sin) {
			while (this->readline(sin));
			return *this;
		}
		
		table& read(const char_type* path) {
			std::basic_ifstream<char_type> fp(path);
			if (!fp.is_open()) throw std::runtime_error("No such file or directory");
			this->read(fp);
			fp.close();
			return *this;
		}
		
		table& read(const string_type& path) {
			return this->read(path.c_str());
		}
		
	private:
		parser parser_;
		
		// invalid copy constructor and self-assign operator
		explicit table(const table& cp);
		table& operator=(const table& cp);
	};
}

#endif // CLX_TABLE_H
