/* ------------------------------------------------------------------------- */
/*
 *  time_duration.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Thu 14 Jun 2007 14:38:02 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TIME_DURATION_H
#define CLX_TIME_DURATION_H

namespace clx {
	typedef long time_value;
	
	namespace time_unit {
		enum {
			minute = 60,	// 60 sec
			hour = 3600,	// 60 * 60 sec
			day = 86400,	// 60 * 60 * 24 sec
			week = 604800	// 60 * 60 * 24 * 7 sec
		};
	};
	
	/* --------------------------------------------------------------------- */
	//  date_duration
	/* --------------------------------------------------------------------- */
	class date_duration {
	public:
		typedef long value_type;
		
		explicit date_duration(value_type y = 0, value_type m = 0, value_type d = 0) :
			years_(y), months_(m), days_(d) {}
		
		date_duration& operator+=(const date_duration& t) {
			years_ += t.years_;
			months_ += t.months_;
			days_ += t.days_;
			return *this;
		}
		
		date_duration& operator-=(const date_duration& t) {
			years_ -= t.years_;
			months_ -= t.months_;
			days_ -= t.days_;
			return *this;
		}
		
		// binary operator
		friend date_duration operator+(date_duration x, const date_duration& y) { return x += y; }
		friend date_duration operator-(date_duration x, const date_duration& y) { return x -= y; }
		friend date_duration operator+(const date_duration& y, date_duration x) { return x += y; }
		friend date_duration operator-(const date_duration& y, date_duration x) { return x -= y; }
		
		// accessor
		value_type years() const { return years_; }
		value_type months() const { return months_; }
		value_type days() const { return days_; }
		
	private:
		value_type years_;
		value_type months_;
		value_type days_;
	};
	
	/* --------------------------------------------------------------------- */
	//  helper_functions
	/* --------------------------------------------------------------------- */
	date_duration years(time_value t) { return date_duration(t); }
	date_duration months(time_value t) { return date_duration(0, t); }
	
	time_value days(time_value t) { return t * time_unit::day; }
	time_value weeks(time_value t) { return t * time_unit::week; }
	time_value hours(time_value t) { return t * time_unit::hour; }
	time_value minutes(time_value t) { return t * time_unit::minute; }
	time_value seconds(time_value t) { return t; }
}

#endif // CLX_TIME_DURATION_H
