/* ------------------------------------------------------------------------- */
/*
 *  timer.h
 *
 *  Copyright (c) 2004 - 2008 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Fri 01 Feb 2008 11:51:16 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TIMER_H
#define CLX_TIMER_H

#ifndef CLX_WIN32
#if defined(_WIN32) || defined(__WIN32__) || defined(WIN32)
#define CLX_WIN32
#endif
#endif // CLX_WIN32

#ifdef CLX_WIN32
#include <windows.h>
#pragma comment(lib, "winmm.lib")
#else
#include <sys/time.h>
#endif

namespace clx {
	namespace detail {
		/* ----------------------------------------------------------------- */
		//  timeval_type
		/* ----------------------------------------------------------------- */
#ifdef CLX_WIN32
		struct use_timeGetTime {
			static double gettime() {
				return static_cast<double>(::timeGetTime()) * 1e-3;
			}
			
			static double precision() { return 1.0 * 1e-3; }
		};
		
		typedef use_timeGetTime timeval_type;
#else
		struct use_gettimeofday {
			static double gettime() {
				struct timeval tv;
				::gettimeofday(&tv, NULL);
				
				double sec = static_cast<double>(tv.tv_sec);
				double usec = static_cast<double>(tv.tv_usec) * 1e-6;
				return sec + usec;
			}
			
			static double precision() { return 1.0 * 1e-6; }
		};
		
		typedef use_gettimeofday timeval_type;
#endif
	};
	
	/* --------------------------------------------------------------------- */
	//  basic_timer
	/* --------------------------------------------------------------------- */
	template <class GetTimePolicy = detail::timeval_type>
	class basic_timer {
	public:
		typedef GetTimePolicy policy;
		
		basic_timer() { this->restart(); }
		
		void restart() {
			start_ = policy::gettime();
			point_ = start_;
		}
		
		double update() {
			double tmp = point_;
			point_ = policy::gettime();
			return point_ - tmp;
		}
		
		double total_elapsed() const { return policy::gettime() - start_; }
		double elapsed() const { return policy::gettime() - point_; }
		double precision() const { return policy::precision(); }
		
	private:
		double start_;
		double point_;
	};
	
	typedef basic_timer<> timer;
}

#endif // CLX_TIMER_H
