/* ------------------------------------------------------------------------- */
/*
 *  udp.h
 *
 *  Copyright (c) 2004 - 2008 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Sun 03 Feb 2008 22:31:03 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_UDP_H
#define CLX_UDP_H

#include <cstdlib>
#include <cstring>
#include <string>
#include "socket.h"
#include "sockaddress.h"
#include "sockstream.h"

namespace clx {
	namespace udp {
		/* ----------------------------------------------------------------- */
		//  basic_socket
		/* ----------------------------------------------------------------- */
		template <
			int Family,
			class CharT = char,
			class Traits = std::char_traits<CharT>
		>
		class basic_socket : public basic_rawsocket<SOCK_DGRAM, Family> {
		public:
			typedef basic_rawsocket<SOCK_DGRAM, Family> rawsocket;
			typedef basic_sockaddress<Family> sockaddress;
			typedef CharT char_type;
			typedef std::basic_string<CharT, Traits> string_type;
			
			// constructor and destructor
			basic_socket() :
				rawsocket(), from_(), to_(), is_bind_(false) {
				this->open();
			}
			
			explicit basic_socket(const char_type* host, int port) :
				rawsocket(), from_(), to_(host, port), is_bind_(false) {
				this->open();
			}
			
			explicit basic_socket(const string_type& host, int port) :
				rawsocket(), from_(), to_(host, port), is_bind_(false) {
				this->open();
			}
			
			explicit basic_socket(int port) :
				rawsocket(), from_(), to_(), is_bind_(false) { 
				this->open();
				this->bind(port);
			}
			
			virtual ~basic_socket() {}
			
			// operator
			int send_to(const char_type* src, int n, const sockaddress& addr) {
				return ::sendto(this->socket(), src, n, 0, addr.base_data(), addr.size());
			}
			
			int send_to(const string_type& src, const sockaddress& addr) {
				return this->send_to(src.c_str(), src.size(), addr);
			}
			
			int send_to(const char_type* src, int n, const char_type* host, int port) {
				sockaddress addr(host, port);
				return this->send_to(src, n, addr);
			}
			
			int send_to(const string_type& src, const string_type& host, int port) {
				sockaddress addr(host, port);
				return this->send_to(src, addr);
			}
			
			int send(const char_type* src, int n) {
				int len = -1;
				
				if (!this->is_bind()) len = this->send_to(src, n, to_);
				else len = this->send_to(src, n, from_);
				return len;
			}
			
			int send(const string_type& src) {
				return this->send(src.c_str(), src.size());
			}
			
			int recv(char_type* dest, int n) {
				if (!this->is_bind()) return -1;
				
				typename sockaddress::inet_type tmp;
				int len = sizeof(tmp);
				std::memset(dest, 0, n);
				
				int ret = ::recvfrom(this->socket(), dest, n, 0, (struct sockaddr*)&tmp, &len);
				if (ret > 0) from_ = tmp;
				return ret;
			}
			
			int bind(int port) {
				to_.assign(port);
				if (::bind(this->socket(), to_.base_data(), to_.size()) == -1) return -1;
				is_bind_ = true;
				return 0;
			}
			
			// accessor
			bool is_bind() const { return is_bind_; }
			const sockaddress& from() const { return from_; }
			const sockaddress& to() const { return to_; }
			
		private:
			sockaddress from_;
			sockaddress to_;
			bool is_bind_;
		};
		
		typedef basic_socket<AF_INET> socket;
		typedef basic_sockstream<socket> sockstream;
	};
}

#endif // CLX_UDP_H
