/* ------------------------------------------------------------------------- */
/*
 *  uri.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Tue 06 Nov 2007 02:17:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_URI_H
#define CLX_URI_H

#include <string>
#include <sstream>
#include "lexical_cast.h"
#include "predicate.h"

namespace clx {
	namespace uri {
		/* ----------------------------------------------------------------- */
		//  is_reserved
		/* ----------------------------------------------------------------- */
		template <class CharT>
		bool is_reserved(CharT c) {
			if (is_alpha()(c) || is_any_of("-_.~ ")(c)) return false;
			return true;
		}
		
		/* ----------------------------------------------------------------- */
		//  encode
		/* ----------------------------------------------------------------- */
		template <class CharT, class Traits>
		std::basic_string<CharT, Traits> encode(const std::basic_string<CharT, Traits>& s) {
			std::basic_ostringstream<CharT, Traits> ss;
			for (unsigned int i = 0; i < s.size(); i++) {
				if (s.at(i) == ' ') ss << '+';
				else if (is_reserved(s.at(i))) {
					ss << '%' << std::hex << static_cast<unsigned int>(s.at(i));
				}
				else ss << s.at(i);
			}
			return ss.str();
		}
		
		/* ----------------------------------------------------------------- */
		//  encode
		/* ----------------------------------------------------------------- */
		template <class CharT>
		std::basic_string<CharT> encode(const CharT* s) {
			std::basic_string<CharT> tmp(s);
			return encode(tmp);
		}
		
		/* ----------------------------------------------------------------- */
		//  decode
		/* ----------------------------------------------------------------- */
		template <class CharT, class Traits>
		std::basic_string<CharT, Traits> decode(const std::basic_string<CharT, Traits>& s) {
			std::basic_ostringstream<CharT, Traits> ss;
			for (unsigned int i = 0; i < s.size(); i++) {
				if (s.at(i) == '%') {
					unsigned int c = lexical_cast<unsigned int>(s.substr(i + 1, 2), std::ios::hex);
					ss << static_cast<CharT>(c);
					i += 2;
				}
				else ss << s.at(i);
			}
			return ss.str();
		}
		
		/* ----------------------------------------------------------------- */
		//  decode
		/* ----------------------------------------------------------------- */
		template <class CharT>
		std::basic_string<CharT> decode(const CharT* s) {
			std::basic_string<CharT> tmp(s);
			return decode(tmp);
		}
	};
}

#endif
